# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['iso18245']

package_data = \
{'': ['*'], 'iso18245': ['data/*']}

setup_kwargs = {
    'name': 'iso18245',
    'version': '1.0.1',
    'description': 'The ISO 18245 Merchant Category Codes database',
    'long_description': '# python-iso18245\n\nA Python implementation of the ISO 18245 Merchant Category Codes database.\n\n## Installation\n\n- `pip install iso18245`\n\n## Usage\n\n```py\n\n>>> import iso18245\n>>> iso18245.get_mcc("5542")\nMCC(range=MCCRange(start=\'5000\', end=\'5599\', description=\'Retail outlets\', reserved=False), iso_description=\'Automated fuel dispensers\', usda_description=\'Automated Fuel Dispensers\', stripe_description=\'Automated Fuel Dispensers\', stripe_code=\'automated_fuel_dispensers\')\n>>> iso18245.get_mcc("3000")\nMCC(range=MCCRange(start=\'3000\', end=\'3999\', description=\'Reserved for private use\', reserved=True), iso_description=\'\', usda_description=\'UNITED AIRLINES\', stripe_description=\'\', stripe_code=\'\')\n>>> iso18245.get_mcc("3000").usda_description\n\'UNITED AIRLINES\'\n>>> iso18245.get_mcc("3000").range\nMCCRange(start=\'3000\', end=\'3999\', description=\'Reserved for private use\', reserved=True)\n>>> iso18245.get_mcc("999999")\nTraceback (most recent call last):\n  …\niso18245.InvalidMCC: 999999\n```\n\n## External links\n\n- [Wikipedia: ISO 18245](https://en.wikipedia.org/wiki/ISO_18245)\n- [ISO Standard 18245:2003](https://www.iso.org/standard/33365.html)\n- [AFNOR: ISO 18245](http://portailgroupe.afnor.fr/public_espacenormalisation/ISOTC68SC7/ISO%2018245.html)\n- [Stripe MCC List](https://stripe.com/docs/issuing/categories)\n- [USDA MCC List (incl. private MCCs)](https://www.dm.usda.gov/procurement/card/card_x/mcc.pdf)\n- [VISA Merchant Data Standards Manual](https://usa.visa.com/content/dam/VCOM/download/merchants/visa-merchant-data-standards-manual.pdf)\n',
    'author': 'Jerome Leclanche',
    'author_email': 'jerome@leclan.ch',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/jleclanche/python-iso18245',
    'packages': packages,
    'package_data': package_data,
    'python_requires': '>=3.5,<4.0',
}


setup(**setup_kwargs)



import csv
import os.path
from collections import namedtuple
from typing import Dict, List

from pkg_resources import resource_filename

ISO_VERSION_YEAR = 2003


class MCCNotFound(KeyError):
	pass


class InvalidMCC(ValueError):
	pass


MCC = namedtuple(
	"MCC",
	(
		"range",
		"iso_description",
		"usda_description",
		"stripe_description",
		"stripe_code",
		"visa_description",
		"visa_req_clearing_name",
	),
)
MCCRange = namedtuple("MCCRange", ("start", "end", "description", "reserved"))

_cached_csv: Dict[str, object] = {}


def _load_csv(path: str):
	full_path = resource_filename("iso18245", os.path.join("data", path))
	if path not in _cached_csv:
		with open(full_path, "r") as f:
			reader = csv.reader(f)
			_cached_csv[path] = list(reader)[1:]

	return _cached_csv[path]


def _find_mcc_in_csv(mcc: str, path: str) -> List[str]:
	for row in _load_csv(path):
		if row[0] == mcc:
			return row[1:]
	return []


def validate_mcc(mcc: str) -> int:
	mcc_as_num = int(mcc)
	if mcc_as_num < 0 or mcc_as_num > 9999:
		raise InvalidMCC(mcc)

	return mcc_as_num


def get_mcc(mcc: str) -> MCC:
	mcc_range = get_mcc_range(mcc)
	found = False
	iso_description = ""
	usda_description = ""
	stripe_description = ""
	stripe_code = ""
	visa_description = ""
	visa_req_clearing_name = ""

	if not mcc_range.reserved:
		data = _find_mcc_in_csv(mcc, "iso18245_official_list.csv")
		if data:
			iso_description, found = data[0], True

	usda_data = _find_mcc_in_csv(mcc, "usda_list.csv")
	if usda_data:
		usda_description, found = usda_data[0], True

	visa_info = _find_mcc_in_csv(mcc, "visa_list.csv")
	if visa_info:
		visa_description, visa_req_clearing_name, found = visa_info[0], visa_info[1], True

	stripe_info = _find_mcc_in_csv(mcc, "stripe_list.csv")
	if stripe_info:
		stripe_description, stripe_code, found = stripe_info[0], stripe_info[1], True

	if not found:
		raise MCCNotFound(mcc)

	return MCC(
		range=mcc_range,
		iso_description=iso_description,
		usda_description=usda_description,
		stripe_description=stripe_description,
		stripe_code=stripe_code,
		visa_description=visa_description,
		visa_req_clearing_name=visa_req_clearing_name,
	)


def get_mcc_range(mcc: str):
	mcc_as_num = validate_mcc(mcc)
	range_data = _load_csv("iso18245_ranges.csv")
	for range_start, range_end, description in range_data:
		start_num, end_num = int(range_start), int(range_end)
		if start_num <= mcc_as_num <= end_num:
			return MCCRange(
				range_start, range_end, description, reserved=description.startswith("Reserved")
			)

		if end_num > mcc_as_num:
			break

	raise RuntimeError(f"Could not find correct MCC range for {mcc} (likely a bug)")


def get_all_mccs_in_range(first: str, last: str) -> List[MCC]:
	first_num = validate_mcc(first)
	last_num = validate_mcc(last)

	lists = [
		_load_csv("iso18245_official_list.csv"),
		_load_csv("stripe_list.csv"),
		_load_csv("usda_list.csv"),
		_load_csv("visa_list.csv"),
	]

	mccs = set()

	for mcc_list in lists:
		for mcc in mcc_list:
			mcc_num = int(mcc[0])
			if mcc_num < first_num:
				continue
			elif mcc_num > last_num:
				break
			mccs.add(mcc[0])

	ret: List[MCC] = []
	for mcc in sorted(mccs):
		ret.append(get_mcc(mcc))

	return ret


def get_all_mccs() -> List[MCC]:
	return get_all_mccs_in_range("0000", "9999")
